﻿using System;
using System.Data;
using System.Drawing;
using System.Windows.Forms;

using Infragistics.Win.UltraWinTree;
using Infragistics.Win;
using System.Diagnostics;

namespace UltraTreePerformance
{
    public partial class Form1 : Form
    {
        private Random random = new Random();
        private int departments = 10;
        private int maxPositions = 10;
        private int minPositions = 2;
        private int employees = 5;
        private int minSalaries = 16;
        private int maxSalaries = 16;

        private Infragistics.Win.Appearance depApp = new Infragistics.Win.Appearance("Department");
        private Infragistics.Win.Appearance posApp = new Infragistics.Win.Appearance("Position");
        private Infragistics.Win.Appearance empApp = new Infragistics.Win.Appearance("Employee");
        private Infragistics.Win.Appearance salApp = new Infragistics.Win.Appearance("Salary");
        private Color[] colors = new Color[] { Color.Red, Color.Blue, Color.Green };

        private EditorWithText editor;

        public Form1()
        {
            this.InitializeComponent();

            this.ultraTree1.ColumnSettings.AllowSorting = Infragistics.Win.DefaultableBoolean.False;
            this.ultraTree1.ColumnSettings.BorderStyleCell = Infragistics.Win.UIElementBorderStyle.Rounded4;
            this.ultraTree1.ColumnSettings.CellAppearance.BorderColor = Color.Transparent;
            this.ultraTree1.ColumnSettings.ActiveCellAppearance.BorderColor = Color.Black;
            this.ultraTree1.Override.CellClickAction = CellClickAction.EditCell;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            depApp.ForeColor = Color.White;
            depApp.BackColor = Color.LightSlateGray;
            posApp.BackColor = Color.AliceBlue;
            empApp.BackColor = Color.SkyBlue;
            salApp.BackColor = Color.DimGray;
        }

        private DataSet GetData()
        {
            DataSet dataSet = new DataSet();

            DataTable dataTableDepartments = new DataTable("Departments");

            dataTableDepartments.Columns.Add("Name", typeof(string));
            dataTableDepartments.Columns.Add("Jan", typeof(int));
            dataTableDepartments.Columns.Add("Feb", typeof(int));
            dataTableDepartments.Columns.Add("Mar", typeof(int));
            dataTableDepartments.Columns.Add("Apr", typeof(int));
            dataTableDepartments.Columns.Add("May", typeof(int));
            dataTableDepartments.Columns.Add("Jun", typeof(int));
            dataTableDepartments.Columns.Add("Jul", typeof(int));
            dataTableDepartments.Columns.Add("Aug", typeof(int));
            dataTableDepartments.Columns.Add("Sep", typeof(int));
            dataTableDepartments.Columns.Add("Oct", typeof(int));
            dataTableDepartments.Columns.Add("Nov", typeof(int));
            dataTableDepartments.Columns.Add("Dec", typeof(int));

            for (int department = 0; department < this.departments; department++)
            {
                string name = string.Format("Department #{0}", department + 1);
                int all = department + 1;
                dataTableDepartments.Rows.Add(new object[] { name, all, all, all, all, all, all, all, all, all, all, all, all });
            }


            DataTable dataTablePositions = new DataTable("Positions");

            dataTablePositions.Columns.Add("Name", typeof(string));
            dataTablePositions.Columns.Add("Jan", typeof(int));
            dataTablePositions.Columns.Add("Feb", typeof(int));
            dataTablePositions.Columns.Add("Mar", typeof(int));
            dataTablePositions.Columns.Add("Apr", typeof(int));
            dataTablePositions.Columns.Add("May", typeof(int));
            dataTablePositions.Columns.Add("Jun", typeof(int));
            dataTablePositions.Columns.Add("Jul", typeof(int));
            dataTablePositions.Columns.Add("Aug", typeof(int));
            dataTablePositions.Columns.Add("Sep", typeof(int));
            dataTablePositions.Columns.Add("Oct", typeof(int));
            dataTablePositions.Columns.Add("Nov", typeof(int));
            dataTablePositions.Columns.Add("Dec", typeof(int));
            dataTablePositions.Columns.Add("Department", typeof(string));


            int positions = this.random.Next(this.minPositions * this.departments, this.maxPositions * this.departments);
            for (int position = 0; position < positions; position++)
            {
                string posName = string.Format("Position #{0}", position + 1);
                int all = position + 1;
                string depName = string.Format("Department #{0}", this.random.Next(0, this.departments + 1) + 1);
                dataTablePositions.Rows.Add(new object[] { posName, all, all, all, all, all, all, all, all, all, all, all, all, depName });
            }


            DataTable dataTableEmployees = new DataTable("Employees");

            dataTableEmployees.Columns.Add("Name", typeof(string));
            dataTableEmployees.Columns.Add("Jan", typeof(int));
            dataTableEmployees.Columns.Add("Feb", typeof(int));
            dataTableEmployees.Columns.Add("Mar", typeof(int));
            dataTableEmployees.Columns.Add("Apr", typeof(int));
            dataTableEmployees.Columns.Add("May", typeof(int));
            dataTableEmployees.Columns.Add("Jun", typeof(int));
            dataTableEmployees.Columns.Add("Jul", typeof(int));
            dataTableEmployees.Columns.Add("Aug", typeof(int));
            dataTableEmployees.Columns.Add("Sep", typeof(int));
            dataTableEmployees.Columns.Add("Oct", typeof(int));
            dataTableEmployees.Columns.Add("Nov", typeof(int));
            dataTableEmployees.Columns.Add("Dec", typeof(int));
            dataTableEmployees.Columns.Add("Position", typeof(string));


            int employees = this.random.Next(this.minPositions * this.departments * this.employees, this.maxPositions * this.departments * this.employees);
            for (int employee = 0; employee < employees; employee++)
            {
                string emplName = string.Format("Employee #{0}", employee + 1);
                int all = employee + 1;
                string posName = string.Format("Position #{0}", this.random.Next(positions + 1));
                dataTableEmployees.Rows.Add(new object[] { emplName, all, all, all, all, all, all, all, all, all, all, all, all, posName });
            }

            DataTable dataTableSalaries = new DataTable("Salaries");

            dataTableSalaries.Columns.Add("Name", typeof(string));
            dataTableSalaries.Columns.Add("Jan", typeof(int));
            dataTableSalaries.Columns.Add("Feb", typeof(int));
            dataTableSalaries.Columns.Add("Mar", typeof(int));
            dataTableSalaries.Columns.Add("Apr", typeof(int));
            dataTableSalaries.Columns.Add("May", typeof(int));
            dataTableSalaries.Columns.Add("Jun", typeof(int));
            dataTableSalaries.Columns.Add("Jul", typeof(int));
            dataTableSalaries.Columns.Add("Aug", typeof(int));
            dataTableSalaries.Columns.Add("Sep", typeof(int));
            dataTableSalaries.Columns.Add("Oct", typeof(int));
            dataTableSalaries.Columns.Add("Nov", typeof(int));
            dataTableSalaries.Columns.Add("Dec", typeof(int));
            dataTableSalaries.Columns.Add("Employee", typeof(string));


            int salaries = this.random.Next(this.minPositions * this.departments * this.employees * this.minSalaries, this.maxPositions * this.departments * this.employees * this.maxSalaries);
            for (int salary = 0; salary < salaries; salary++)
            {
                string salName = string.Format("Salary #{0}", salary + 1);
                int all = salary + 1;
                string emplName = string.Format("Employee #{0}", this.random.Next(employees + 1));
                dataTableSalaries.Rows.Add(new object[] { salName, all, all, all, all, all, all, all, all, all, all, all, all, emplName });
            }

            dataSet.Tables.Add(dataTableDepartments);
            dataSet.Tables.Add(dataTablePositions);
            dataSet.Tables.Add(dataTableEmployees);
            dataSet.Tables.Add(dataTableSalaries);

            dataSet.Relations.Add("PositionsToDepartments", dataTableDepartments.Columns["Name"], dataTablePositions.Columns["Department"], false);
            dataSet.Relations.Add("EmployeesToPositions", dataTablePositions.Columns["Name"], dataTableEmployees.Columns["Position"], false);
            dataSet.Relations.Add("SalariesToEmployees", dataTableEmployees.Columns["Name"], dataTableSalaries.Columns["Employee"], false);

            return dataSet;
        }

        private void ultraTree1_InitializeDataNode(object sender, InitializeDataNodeEventArgs e)
        {
            if (this.checkBox1.Checked)
            {
                foreach (UltraTreeNodeCell cell in e.Node.Cells)
                {
                    cell.Appearance.BackColor = this.colors[this.random.Next(3)];
                }
            }
            else
            {
                switch (e.Node.Level)
                {
                    case 0:
                        e.Node.Override.NodeAppearance = this.depApp;
                        break;
                    case 1:
                        e.Node.Override.NodeAppearance = this.posApp;
                        break;
                    case 2:
                        e.Node.Override.NodeAppearance = this.empApp;
                        break;
                    case 3:
                        e.Node.Override.NodeAppearance = this.salApp;
                        //e.Node.AllowCellEdit = AllowCellEdit.Full;
                        foreach (var cell in e.Node.Cells)
                        {
                            cell.AllowEdit = AllowCellEdit.Full;
                        }
                        break;
                    default:
                        break;
                }
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            DataSet dataSet = this.GetData();
            this.ultraTree1.ColumnSettings.AutoGenerateColumnSets = true;
            this.ultraTree1.SynchronizeCurrencyManager = false;
            this.ultraTree1.ViewStyle = ViewStyle.OutlookExpress;
            this.ultraTree1.ColumnSettings.AutoFitColumns = AutoFitColumns.ResizeAllColumns;
            this.ultraTree1.DataSource = null;
            this.ultraTree1.DataSource = dataSet;
            this.ultraTree1.DataMember = "Departments";
        }

        private void ultraTree1_AfterCellEnterEditMode(object sender, AfterCellEnterEditModeEventArgs e)
        {
            if (this.editor == null)
            {
                this.editor = (EditorWithText)e.Cell.EditorResolved;
                this.editor.ValueChanged += editor_ValueChanged;
            }
        }

        private void editor_ValueChanged(object sender, EventArgs e)
        {
            EditorWithText editor = sender as EditorWithText;
            if (editor == null)
            {
                return;
            }

            double currentCellValue = 0;
            double.TryParse(editor.Value.ToString(), out currentCellValue);

            UltraTreeNodeCell cell = this.ultraTree1.ActiveCell;
            if (cell == null || cell.IsInEditMode == false)
            {
                return;
            }

            if (cell.Column.Key == "Jan")
            {
                UltraTreeNodeCell sumCell = cell.Node.Parent.Cells["Jan"];
                double sumValue = 0;
                foreach (UltraTreeNode siblingNode in sumCell.Node.Nodes)
                {
                    if (siblingNode.Cells["Jan"] == cell)
                    {
                        sumValue += currentCellValue;
                        continue;
                    }

                    sumValue += double.Parse(siblingNode.Cells["Jan"].Value.ToString());
                }

                sumCell.Value = sumValue;
            }

        }

        private void ultraTree1_AfterCellExitEditMode(object sender, AfterCellExitEditModeEventArgs e)
        {
            if (this.editor != null)
            {
                this.editor.ValueChanged -= editor_ValueChanged;
                this.editor = null;
            }

            var newData = GetData();
            var oldData = (DataSet)this.ultraTree1.DataSource;

            oldData.Tables["Departments"].Load(newData.Tables["Departments"].CreateDataReader(), LoadOption.Upsert);
            oldData.Tables["Positions"].Load(newData.Tables["Positions"].CreateDataReader(), LoadOption.Upsert);
            oldData.Tables["Employees"].Load(newData.Tables["Employees"].CreateDataReader(), LoadOption.Upsert);
            oldData.Tables["Salaries"].Load(newData.Tables["Salaries"].CreateDataReader(), LoadOption.Upsert);

        }

        private void ultraTree1_Paint(object sender, PaintEventArgs e)
        {
            Debug.WriteLine(e.ClipRectangle);
        }

        private void buttonAddRow_Click(object sender, EventArgs e)
        {
            UltraTreeNodeCell cell = this.ultraTree1.ActiveCell;
            if (cell == null || cell.Node.Level == 0)
            {
                return;
            }

            DataSet ds = (DataSet)this.ultraTree1.DataSource;
            string key = ((DataRowView)(cell.Node.ListObject)).Row.Table.TableName;
            DataTable dt = ds.Tables[key];



            string salName = this.random.Next(0, 4000).ToString();
            int all = this.random.Next(0, 4000);
            string emplName = cell.Node.Parent.Cells["Name"].Value.ToString();

            dt.Rows.Add(new object[] { salName, all, all, all, all, all, all, all, all, all, all, all, all, emplName });

        }
    }
}
